@@ -1,7 +1,7 @@
 ###################################################################################################
 ###                                                                                                            
 ### FUNCION PARA CARACTERIZACION DE VARIABLES EN CLUSTER  - INGLS                                             
-### ltima revisin: abril 22/08                                                                    
+### ltima revisin: julio 22/15                                                                   
 ### Elaborado por: Pedro Cesar del Campo Neira     
 ###    Revisado por: Campo Elas Pardo                                                                         
 ###    Traduccin de mensajes al ingls Campo Elas Pardo
@@ -10,14 +10,14 @@
 ###                                                                                                            
 ### cluster.carac( tabla  := objeto data.frame con variables a caracterizar de un solo tipo                    
 ###                (continuas o discretas)                                                                     
-###        clase  := vector que determina la particion de la tabla ( as.factor(clase)=TRUE )                   
+###        class  := vector que determina la particion de la tabla ( as.factor(class)=TRUE )                   
 ###        tipo.v := tipo de variables (continuas o discretas)                                                 
 ###        sign   := Nivel de significancia para la prueba estadistica... para eliminar V.test's               
 ###      )                                                                                                     
 ###                                                                                                            
 ###################################################################################################
 
-cluster.carac<-function( tabla , clase , tipo.v="d" , v.lim = 2 ){
+cluster.carac<-function(tabla,class,tipo.v="d",v.lim=2,dn=3,dm=3){
 
     if (!inherits(tabla, "data.frame")) 
         stop("The first argument must be an object of class data.frame") # control de objeto
@@ -49,33 +49,26 @@ cluster.carac<-function( tabla , clase , tipo.v="d" , v.lim = 2 ){
              Global   <- 100*nj/n                     ##  % de mod_j en n 
 
              ##  probabilidad hipergeometrica y valor test
-             p.lim=rep(pnorm(v.lim),length(nj))
-             prob <- matrix(0.5, length(nj), 1)
+             # programado por CEPT julio 2015
 
-             #los que se caracterizan por ausencia
-             Prob.AcI <- phyper(njk, nj, n - nj, nk)
-             prob[Prob.AcI < 1-p.lim] <- Prob.AcI[Prob.AcI < 1-p.lim]
-
-             #si la distribucin hipergeomtrica toma el valor cero con probabilidad mayor..
-             #a 1-p.lim, entonces no se puede caracterizar por ausencia, por lo tanto se..
-             #aumenta el umbral para caracterizar por presencia de la caracterstica.
-             p.lim[dhyper(0, nj, n - nj, nk) >= 1-p.lim] <- 2*pnorm(v.lim)-1
-
-             #los que se caracterizan por presencia
-             Prob.AcD <- phyper(njk, nj, n - nj, nk) - dhyper(njk, nj, n - nj, nk)
-             prob[Prob.AcD > p.lim] <- Prob.AcD[Prob.AcD > p.lim]
-
-             V.test <- qnorm(prob)
-
-             SALIDA <- data.frame(Test.Value=round(V.test,3), p.Value=round(prob,3), 
+             prob <- matrix(NA, length(nj), 1) # 0.5 -> NA CEPT
+             
+             
+             prob <- phyper(njk, nj, n - nj, nk)
+             prob[mod.clas>=Global]<-phyper(njk-1,nj,n-nj,nk,lower.tail=FALSE)[mod.clas>Global]
+             #valores test
+             V.test <- qnorm(prob/2) 
+             V.test[mod.clas>Global]<-qnorm(prob/2,lower.tail=FALSE)[mod.clas>=Global] 
+             
+             SALIDA <- data.frame(Test.Value=round(V.test,dn), p.Value=round(prob,dn), 
              Class.Cat=round(clas.mod,1), Cat.Class=round(mod.clas,1),                           
              Global=round(Global,1), Weight = nj)                                                    
              rownames(SALIDA) <- rownames(data.frame(nj))                              
-             SALIDA <- subset(SALIDA, prob != 0.5)                                     
+             SALIDA <- subset(SALIDA, abs(V.test) >= v.lim)                                     
              SALIDA <- SALIDA[order(SALIDA$Test.Value, decreasing = TRUE),]                
              return(SALIDA)
              }
-        return(by(tabla, clase, interno))
+        return(by(tabla, class, interno))
     }
 # Fin variables discretas 
        
@@ -99,12 +92,12 @@ cluster.carac<-function( tabla , clase , tipo.v="d" , v.lim = 2 ){
              V.test   <- ( mean.Xk - mean.X ) / sqrt(S2.Xk)    ## valores.test cluster_k
              
              ##-------------------------------
-             SALIDA   <- data.frame(Test.Value=round(V.test,3),Class.Mean=mean.Xk,Frequency=nk,Global.Mean=mean.X)      
+             SALIDA   <- data.frame(Test.Value=round(V.test,dn),Class.Mean=round(mean.Xk,dm),Frequency=nk,Global.Mean=round(mean.X,dm))      
              ## SALIDA
  
              rownames(SALIDA)<-names(c.tabla)                   ## etiquetas variables 
              SALIDA<-SALIDA[-1,]                                ## Eliminacion variable falsa
-             SALIDA <- subset(SALIDA, abs(SALIDA$Test.Value) > v.lim )    
+             SALIDA <- subset(SALIDA, abs(SALIDA$Test.Value) >= v.lim )    
                                                                 ## salida de no representativos
              SALIDA <- SALIDA[order(SALIDA$Test.Value , decreasing = TRUE),]   ## ordena por V.test
  
@@ -113,7 +106,7 @@ cluster.carac<-function( tabla , clase , tipo.v="d" , v.lim = 2 ){
                return(SALIDA)
             }
 
-  return( by(tabla,clase,interno) ) # orden "by" salida para todos los cluster
+  return( by(tabla,class,interno) ) # orden "by" salida para todos los cluster
 
  }# Fin variables continuas    
 
@@ -133,35 +126,32 @@ cluster.carac<-function( tabla , clase , tipo.v="d" , v.lim = 2 ){
              clas.mod <- 100*njk/nj                   ##  % de cluster_k en mod_j
              mod.clas <- 100*njk/nk                   ##  % de mod_j en cluster_k 
              Global   <- 100*nj/n                     ##  % de mod_j en n 
-
-                   ##  probabilidad hipergeometrica y valor test                            
-             p.lim=rep(pnorm(v.lim),length(nj))                                                    
-             prob <- matrix(0.5, length(nj), 1)                                                    
-                                                                                            
-             #los que se caracterizan por ausencia                                                 
-             Prob.AcI <- phyper(njk, nj, n - nj, nk)                                               
-             prob[Prob.AcI < 1-p.lim] <- Prob.AcI[Prob.AcI < 1-p.lim]                              
-                                                                                            
-             #si la distribucin hipergeomtrica toma el valor cero con probabilidad mayor..       
-             #a 1-p.lim, entonces no se puede caracterizar por ausencia, por lo tanto se..         
-             #aumenta el umbral para caracterizar por presencia de la caracterstica.              
-             p.lim[dhyper(0, nj, n - nj, nk) >= 1-p.lim] <- 2*pnorm(v.lim)-1                       
-                                                                                            
-             #los que se caracterizan por presencia                                                
-             Prob.AcD <- phyper(njk, nj, n - nj, nk) - dhyper(njk, nj, n - nj, nk)                 
-             prob[Prob.AcD > p.lim] <- Prob.AcD[Prob.AcD > p.lim]                                  
-                                                                                            
-             V.test <- qnorm(prob)                                                                 
-             SALIDA <- data.frame(Test.Value=round(V.test,3), p.Value=round(prob,3), 
+             
+             # se coloca la de variables categricas CEPT
+             
+             ##  probabilidad hipergeometrica y valor test
+             # programado por CEPT julio 2015
+             p.lim<-pnorm(-v.lim)
+             prob <- matrix(NA, length(nj), 1) # 0.5 -> NA CEPT
+             
+             
+             p.lim=rep(pnorm(-v.lim),length(nj)) # v.lim -> -vlim CEPT
+             prob <- phyper(njk, nj, n - nj, nk)
+             prob[mod.clas>Global]<-phyper(njk-1,nj,n-nj,nk,lower.tail=FALSE)[mod.clas>Global]
+             #valores test
+             V.test <- qnorm(prob/2) 
+             V.test[mod.clas>Global]<-qnorm(prob/2,lower.tail=FALSE)[mod.clas>Global] 
+             
+             SALIDA <- data.frame(Test.Value=round(V.test,dn), p.Value=round(prob,dn), 
              Class.Cat=round(clas.mod,1), Cat.Class=round(mod.clas,1),                           
              Global=round(Global,1), Weight = nj)                                                    
              rownames(SALIDA) <- rownames(data.frame(nj))                              
-             SALIDA <- subset(SALIDA, prob != 0.5)                                     
+             SALIDA <- subset(SALIDA,  abs(SALIDA$Test.Value) >= v.lim)                                     
              SALIDA <- SALIDA[order(SALIDA$Test.Value, decreasing = TRUE),]                
              return(SALIDA)                                                                        
              }
 
-   return( by(tabla,clase,interno) ) # orden "by" salida para todos los cluster 
+   return( by(tabla,class,interno) ) # orden "by" salida para todos los cluster 
 
  }# End of counting or frequency variables 
 }
